//: attrOLView.TDH.cpp
//////////////////////////////////////////////////////////////////////////////
// TDH = Typed Data Hander

#include <TypeConstants.h>
#include <Mime.h>
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "attrOLView.h"
#include "myItem.h"


//////////////////////////////////////////////////////////////////////////////
// AttrOLView::AddAttrString
/*
	This should eventually allow pretty type names...
*/

void AttrOLView::AddAttrString(
	type_code     type, 
	size_t        size,
	const char  * name,
	const char  * value)
{
//puts("IN!");
	static char buf[2048]="";
	static char typeChars[6]="";
	*((uint32*)typeChars)=type;
	typeChars[4]='\0';
	
	sprintf(buf, "\2b(\2D'%s'%5u\2b)\2D %-22s \2b:\2p %s", 
		typeChars, size, name, value);
//puts(buf);
	AddItem(new MyStringItem(buf, CurLevel(), false));
//puts("OUT!");
}

#define MISSIZE_CHECK(TYPE) \
{  \
	if (datalen != sizeof(TYPE))  \
	{   \
		sprintf(minibuf, "\2b<< MIS-SIZED DATA >>\2p     "   \
			"expected %u bytes,  found %u bytes", sizeof(TYPE), datalen); \
		AddAttrString(origType, datalen, name, minibuf);	   \
		return;  \
	}             \
}


//////////////////////////////////////////////////////////////////////////////
// AttrOLView::HandleTypedData
/*
	Assumes "data" points at the type signified by "type", 
	is of "datalen" bytes, and has a 0 byte at data[datalen]
*/

void AttrOLView::HandleTypedData(
	const void  * data, 
	size_t        datalen,
	type_code     type,
	const char  * name,
	type_code     origType)
{
//if (type=='MICN') printf("\nAttrOLView::HandleTypedData in for 'MICN'\n");

  static char bigbuf[16384]="";  
  static char minibuf[2048]="";  
  bigbuf[0] = minibuf[0] = '\0';
  const color_map * cmap = system_colors();	
  long ii=0, pos=0;

  if(!data)    // ok, we messed up, but don't crash the Tracker!
  	return;

  if (!origType)
  	origType = type;

  switch(type)
  {
	case B_ANY_TYPE 			:
	{
		AddAttrString(origType, datalen, name, "<< ANY TYPE >>");
		DoRaw(data, datalen);	
	} return;
		
	case B_BOOL_TYPE 			:
	{
		MISSIZE_CHECK(bool)
		sprintf(minibuf, *(bool*)data ? "true" : "false");
		AddAttrString(origType, datalen, name, minibuf);
	} return;

	case B_DOUBLE_TYPE 		:
	{
		MISSIZE_CHECK(double)
		sprintf(minibuf, "%.10f", *(double*)data);
		AddAttrString(origType, datalen, name, minibuf);		
	} return;

	case B_FLOAT_TYPE 			:
	{
		MISSIZE_CHECK(float)
		sprintf(minibuf, "%f", *(float*)data);
		AddAttrString(origType, datalen, name, minibuf);		
	} return;
	
	case B_INT64_TYPE 			:
	{
		MISSIZE_CHECK(int64)
		char array[10]="";
		*((int64*)array) = *(int64*)data;
		array[8]=0;
	
		sprintf(minibuf, "\2f%Ld   \2b0x\2f%08LX   '%s'", 
			*(int64*)data, *(int64*)data, array);

		AddAttrString(origType, datalen, name, minibuf);		
	} return;
	
	case B_INT32_TYPE 			:
	{
		MISSIZE_CHECK(int32)
		char array[10]="";
		*((int32*)array) = *(int32*)data;
		array[4]=0;
	
		sprintf(minibuf, "\2f%ld   \2b0x\2f%08lX   '%s'", 
			*(int32*)data, *(int32*)data, array);

		AddAttrString(origType, datalen, name, minibuf);		
	} return;
	
	case B_INT16_TYPE 			:
	{
		MISSIZE_CHECK(int16)
		char array[10]="";
		*((int16*)array) = *(int16*)data;
		array[2]=0;

		sprintf(minibuf, "\2f%hd   \2b0x\2f%04hX   '%s'", 
			*(int16*)data, *(int16*)data, array);

		AddAttrString(origType, datalen, name, minibuf);		
	} return;
	
	case B_INT8_TYPE 			:
	{
		MISSIZE_CHECK(int8)
		sprintf(minibuf, "\2f%hd   \2b0x\2f%02hX  '%c'", 
			(int16) *(int8*)data,  (int16) *(int8*)data,
			*(int8*)data  );

		AddAttrString(origType, datalen, name, minibuf);		
	} return;
	
	case B_UINT64_TYPE 			:
	{
		MISSIZE_CHECK(uint64)
		char array[10]="";
		*((uint64*)array) = *(uint64*)data;
		array[8]=0;
	
		sprintf(minibuf, "\2f%Lu   \2b0x\2f%016LX   '%s'", 
			*(uint64*)data, *(uint64*)data, array);

		AddAttrString(origType, datalen, name, minibuf);		
	} return;
	
	case B_UINT32_TYPE 			:
	{
		MISSIZE_CHECK(uint32)
		char array[10]="";
		*((uint32*)array) = *(uint32*)data;
		array[4]=0;
	
		sprintf(minibuf, "\2f%lu   \2b0x\2f%08lX   '%s'", 
			*(uint32*)data, *(uint32*)data, array);

		AddAttrString(origType, datalen, name, minibuf);		
	} return;
	
	case B_UINT16_TYPE 			:
	{
		MISSIZE_CHECK(uint16)
		char array[10]="";
		*((uint16*)array) = *(uint16*)data;
		array[2]=0;
	
		uint32 it = (uint32) *(uint16*)data;
		sprintf(minibuf, "\2f%hu   \2b0x\2f%04hX   '%s'", it, it, array);

		AddAttrString(origType, datalen, name, minibuf);		
	} return;

	case B_UINT8_TYPE 			:
	{	
		MISSIZE_CHECK(uint8)
		sprintf(minibuf, "\2f%hu   \2b0x\2f%02hX  '%c'", 
			(uint16) *(uint8*)data,  (uint16) *(uint8*)data,
			*(uint8*)data  );

		AddAttrString(origType, datalen, name, minibuf);		
	} return;
	

	case B_ASCII_TYPE 			:  
	{
		AddAttrString(origType, datalen, name, (char*)data);
	} return;
		
	case B_STRING_TYPE 			:
	{
		AddAttrString(origType, datalen, name, (char*)data);
	} return;
	
	case B_CHAR_TYPE 			:
	{
		MISSIZE_CHECK(char)
		sprintf(minibuf, "'%c'", *(char*)data);
		AddAttrString(origType, datalen, name, minibuf);		
	} return;
	 
	
	case B_MESSAGE_TYPE		:
	{
		AddAttrString(origType, datalen, name, "<< Be MESSAGE >>");	
		DoRaw(data, datalen);		
	} return;
	
	case B_MESSENGER_TYPE		:
	{
		AddAttrString(origType, datalen, name, "<< Be MESSENGER >>");		
		DoRaw(data, datalen);		
	} return;
	
	case B_MIME_TYPE			:
	{
		AddAttrString(origType, datalen, name, "<< MIME TYPED DATA >>");		
		DoRaw(data, datalen);		
	} return;
	
	case B_MIME_STRING_TYPE		:
	{
		const char * str = (const char*)data;
		int offset = strchr(str, '/')-str;
		strncpy(bigbuf, str, offset);
		bigbuf[offset] = 0;
		strcat (bigbuf, "\2b/\2p");
		strcat (bigbuf, str+offset+1);
		AddAttrString(origType, datalen, name, bigbuf);
	} return;
		
	case B_OBJECT_TYPE 			:
	{
		AddAttrString(origType, datalen, name, 
			"<< OBJECT (of unknown type) >>");		
		DoRaw(data, datalen);		
	} return;
	
	case B_OFF_T_TYPE 			:
	{
		MISSIZE_CHECK(off_t)
		// this is a long long
		sprintf(minibuf, "Offset: %Ld", 
			(int64) (*(off_t*)data));
		AddAttrString(origType, datalen, name, minibuf);
	} return;
	
	case B_PATH_TYPE			:
	{
		// Its not clear what type the data will really be!
		// In a BMessage its supposed to be a flattened BPath (really 
		// an entry_ref). But for an attribute, who knows how it was saved?
		// There must be a null term string in there some where...
		sprintf(bigbuf, "Path: \"%s\"  (maybe...??)", (char*)data);	
		AddAttrString(origType, datalen, name, bigbuf);
	} return;
		
	case B_PATTERN_TYPE 		:	
	{
		MISSIZE_CHECK(pattern)
		
		pattern ptrn = *(pattern*)data;
		pos = 0;
		for (ii=0; ii<(sizeof(pattern)*8); ii++)
		{
			sprintf(minibuf+pos, "%02hX ", (uint16)(uint8)ptrn.data[ii] );
			pos += 3;
		}
		
		/*
		sprintf(minibuf+pos, " : ");
		pos += 3;
		for (ii=0; ii<(sizeof(pattern)*8); ii++, pos++)
		{	
			if (0==(ii%8))
				minibuf[pos++] = ' ';
			minibuf[pos] = (((ptrn.data[ii/8]) << (ii%8)) & 1) ? '*':'.';	
		}	
		*/
		AddAttrString(origType, datalen, name, minibuf);
	} return;
	
	case B_POINTER_TYPE 		:	
	{
		MISSIZE_CHECK(void*)
		// in an attribute?? Whatever...
		sprintf(minibuf, "%p", *(void**)data);
		AddAttrString(origType, datalen, name, minibuf);
	} return;
	
	case B_RAW_TYPE 			:	
	{	
		AddAttrString(origType, datalen, name, "<< RAW DATA >>");		
		DoRaw(data, datalen);
	} return;
	
	case B_POINT_TYPE 			:
	{
		MISSIZE_CHECK(BPoint)
		BPoint pt = *(BPoint*)data;
		sprintf(minibuf, "\2b(\2p %.0f\2b,\2p %.0f \2b)", pt.x, pt.y );
		AddAttrString(origType, datalen, name, minibuf);
	} return;
	
	case B_RECT_TYPE 			:
	{
		MISSIZE_CHECK(BRect)

		BRect & rect = *(BRect*)data;
		sprintf(minibuf, 
			"\2b( (\2p%.0f\2b, \2p%.0f\2b),  (\2p%.0f\2b, \2p%.0f\2b) )"
			"      W:\2p %0.f   \2bH:\2p %0.f", 
			rect.left, rect.top,   rect.right, rect.bottom,
			rect.Width(), rect.Height() );

		AddAttrString(origType, datalen, name, minibuf);
	} return;
	
	case B_REF_TYPE 			:	
	{
		MISSIZE_CHECK(entry_ref)
		
		entry_ref & ref = *(entry_ref*)data;
		BPath path;
		BEntry entry(&ref,false);	// don't traverse
		if (entry.InitCheck() == B_OK) 
		{
			entry.GetPath(&path);
			strcpy(minibuf, path.Path());
		}
		else
			strcpy(minibuf, " << NON-STATABLE REF PATH >> ");
			
		sprintf( bigbuf, 
			"%s        (name: '%s'    device: %ld    dir inode: %Ld)",
			minibuf, ref.name, ref.device, ref.directory);

		AddAttrString(origType, datalen, name, bigbuf);
	} return;
	
		
	case B_MONOCHROME_1_BIT_TYPE:
	{
		AddAttrString(origType, datalen, name, 
			"<< 1 BIT MONOCHROME BITMAP >>");
	} return;
	
	case B_GRAYSCALE_8_BIT_TYPE:
	{
		AddAttrString(origType, datalen, name, 
			"<< 8 BIT GRAYSCALE BITMAP >>");
	} return;
	
	case B_COLOR_8_BIT_TYPE 	:
	{
		if (datalen==1024 && 0==strncmp("BEOS:L:STD_ICON", name, 15))
			HandleTypedData (data, datalen, 'ICON', name, origType);
		else if (datalen==256 && 0==strncmp("BEOS:M:STD_ICON", name, 15))
			HandleTypedData (data, datalen, 'MICN', name, origType);
		else
			AddAttrString(origType, datalen, name, 
				"<< 8 BIT COLOR BITMAP >>");
	} return;

	case B_RGB_32_BIT_TYPE 		:
	{
		AddAttrString(origType, datalen, name, 
			"<< 32 BIT COLOR BITMAP >>");
	} return;

	case 'ICON' :  // 32x32 8 bit bitmap
	{
		AddAttrString(origType, datalen, name, 
			"<< LARGE ICON >>");
			
		EnterSubLevel();
		AddItem(new ImageItem(
			32, 32, B_COLOR_8_BIT, data, CurLevel(), false));
		LeaveSubLevel();
	} return;
	
	case 'MICN' :  // 32x32 8 bit bitmap
	{
		AddAttrString(origType, datalen, name, 
			"<< MINI ICON >>");
		
		EnterSubLevel();
		
		ImageItem * item = new ImageItem(
			16, 16, B_COLOR_8_BIT, data, CurLevel(), false);
		AddItem(item);

		LeaveSubLevel();
	} return;
	
	case B_RGB_COLOR_TYPE 		:
	{
		MISSIZE_CHECK(rgb_color)
		
		rgb_color color = *(rgb_color*)data;
		uint16 rr = (uint16) color.red;
		uint16 gg = (uint16) color.green;
		uint16 bb = (uint16) color.blue;
		uint16 aa = (uint16) color.alpha;
		
		sprintf(minibuf, 
			"\2bR:\2p %hu   \2bG:\2p %hu   \2bB:\2p %hu   \2bA:\2p %hu    "
			" (R:%02hX  G:%02hX  B:%02hX  A:%02hX)",
			rr,gg,bb,aa, rr,gg,bb,aa);

		AddAttrString(origType, datalen, name, minibuf);			
	} return;
	
	case B_SIZE_T_TYPE	 		:
	{
		MISSIZE_CHECK(size_t)
		sprintf(minibuf, "%lu", *(size_t*)data);
		AddAttrString(origType, datalen, name, minibuf);
	} return;
	
	case B_SSIZE_T_TYPE	 		:
	{
		MISSIZE_CHECK(ssize_t)
		sprintf(minibuf, "%ld", *(ssize_t*)data);
		AddAttrString(origType, datalen, name, minibuf);
	} return;
		
	case B_TIME_TYPE 			:
	{
		if (datalen == sizeof(bigtime_t))
		{	
			bigtime_t time  = *(bigtime_t*)data;
			time_t    stime = (time_t)(time/(1000*1000));
			sprintf(minibuf, "%ld.%Lu seconds  (%Lu µs)   %s", 
				stime, time%(1000*1000), time, ctime(&stime)   );  
		}
		else if (datalen == sizeof(time_t))
		{
			time_t time = *(time_t*)data;
			sprintf(minibuf, "%lu seconds    %s", 
				time, ctime(&time));  
		}
		else
			strcpy(minibuf, "<< TIME (length unrecognized/unknown >>");

		// ctime returns a string with a newline at the end
		// which shows up as a block in the BStringView.
		// Null it.  
		if (minibuf[strlen(minibuf)-1] == '\n')
			minibuf[strlen(minibuf)-1] = '\0';

		AddAttrString(origType, datalen, name, minibuf);		
	} return;
	
	
	default:
	{
		AddAttrString(origType, datalen, name, 
			"<< UNKNOWN TYPE >>");
		DoRaw(data, datalen);	
	} return;
  }
  
  // unreachable
}


//////////////////////////////////////////////////////////////////////////////
// AttrOLView::doRaw

void AttrOLView::DoRaw(const void * data, size_t datalen)
{
	uint8 * raw = (uint8*)data;
	static char buf[1024]="";  
	//long ii, pos;

	EnterSubLevel();

	Hexprint((const char*)data, datalen);

	/*
	for (ii=0, pos=0; ii<datalen; ii++)
	{	
		if (0==(ii%4) && 0!=(ii%16))
			buf[pos++] = ' ';
		sprintf(buf+pos, "%02lX", (uint32)(raw[ii]) );				
		pos += 2;
		
		if (0==(ii%16))
		{
			buf[pos] = '\0';				
			AddItem(new BStringItem(buf, CurLevel(), false));
			pos=0;
		}
	}
	*/
	
	LeaveSubLevel();
	return;
}


//////////////////////////////////////////////////////////////////////////////
// AttrOLView::TypeCodeToString
/*
	Writes a string rep of type_ into buf_.
	buf_ assumed to be long enough to hold type name (64B is plenty).
	returns false if type_ not found (buf_ unmodified)
*/

bool AttrOLView::TypeCodeToString(
	type_code type_, 
	char * buf_,
	bool prettify)
{
	for (long ii=1; typeNames[ii].Type() != 0; ii++)
		if (typeNames[ii].IsType(type_))
		{
			if (prettify)
			{
				// hack off "B_" w/              +2
				strcpy(buf_, typeNames[ii].Name()+2);
				
				// hack off "_TYPE":  -5
				int len = strlen(buf_)-5;
				buf_[len] = '\0';

				// leave     1st  letter upcase
				for (long jj=1; jj<len; jj++)
				{
					buf_[jj] = tolower(buf_[jj]);
					if (buf_[jj] == '_')
						buf_[jj] = ' ';
				}
			}
			else
				strcpy(buf_, typeNames[ii].Name());
			return true; 
		}
		
	return false;
}	

//////////////////////////////////////////////////////////////////////////////

#define MAKETNP(xXx) TypeNamePair(xXx, #xXx)

TypeNamePair typeNames[] = 
{
	TypeNamePair(0, "<< UNKNOWN >>"),
	MAKETNP(B_ANY_TYPE),
	MAKETNP(B_ASCII_TYPE),
	MAKETNP(B_BOOL_TYPE),
	MAKETNP(B_CHAR_TYPE),
	MAKETNP(B_COLOR_8_BIT_TYPE),
	MAKETNP(B_DOUBLE_TYPE), 		
	MAKETNP(B_FLOAT_TYPE), 			
	MAKETNP(B_GRAYSCALE_8_BIT_TYPE),
	MAKETNP(B_INT64_TYPE), 			
	MAKETNP(B_INT32_TYPE), 			
	MAKETNP(B_INT16_TYPE), 			
	MAKETNP(B_INT8_TYPE), 			
	MAKETNP(B_MESSAGE_TYPE),		
	MAKETNP(B_MESSENGER_TYPE),		
	MAKETNP(B_MIME_TYPE),	
	MAKETNP(B_MIME_STRING_TYPE),	    // 'MIMS', from Mime.h
	MAKETNP(B_MONOCHROME_1_BIT_TYPE),
	MAKETNP(B_OBJECT_TYPE), 			
	MAKETNP(B_OFF_T_TYPE), 			
	MAKETNP(B_PATH_TYPE),				
	MAKETNP(B_PATTERN_TYPE), 			
	MAKETNP(B_POINTER_TYPE), 			
	MAKETNP(B_POINT_TYPE), 			
	MAKETNP(B_RAW_TYPE), 				
	MAKETNP(B_RECT_TYPE), 			
	MAKETNP(B_REF_TYPE), 				
	MAKETNP(B_RGB_32_BIT_TYPE), 		
	MAKETNP(B_RGB_COLOR_TYPE), 		
	MAKETNP(B_SIZE_T_TYPE),	 		
	MAKETNP(B_SSIZE_T_TYPE),	 		
	MAKETNP(B_STRING_TYPE), 			
	MAKETNP(B_TIME_TYPE), 			
	MAKETNP(B_UINT64_TYPE),			
	MAKETNP(B_UINT32_TYPE),			
	MAKETNP(B_UINT16_TYPE), 			
	MAKETNP(B_UINT8_TYPE), 	
	TypeNamePair('ICON', "LARGE_ICON"),		
	TypeNamePair('MICN', "MINI_ICON"),		
	TypeNamePair(0, "<< END OF LIST >>")
};



/*
	B_ANY_TYPE 			
	B_ASCII_TYPE 			
	B_BOOL_TYPE 			
	B_CHAR_TYPE 			
	B_COLOR_8_BIT_TYPE 	
	B_DOUBLE_TYPE 		
	B_FLOAT_TYPE 			
	B_GRAYSCALE_8_BIT_TYPE
	B_INT64_TYPE 			
	B_INT32_TYPE 			
	B_INT16_TYPE 			
	B_INT8_TYPE 			
	B_MESSAGE_TYPE		
	B_MESSENGER_TYPE		
	B_MIME_TYPE				
	B_MONOCHROME_1_BIT_TYPE
	B_OBJECT_TYPE 			
	B_OFF_T_TYPE 			
	B_PATH_TYPE				
	B_PATTERN_TYPE 			
	B_POINTER_TYPE 			
	B_POINT_TYPE 			
	B_RAW_TYPE 				
	B_RECT_TYPE 			
	B_REF_TYPE 				
	B_RGB_32_BIT_TYPE 		
	B_RGB_COLOR_TYPE 		
	B_SIZE_T_TYPE	 		
	B_SSIZE_T_TYPE	 		
	B_STRING_TYPE 			
	B_TIME_TYPE 			
	B_UINT64_TYPE			
	B_UINT32_TYPE			
	B_UINT16_TYPE 			
	B_UINT8_TYPE 			
*/




