//: attrOLView.hexprint.cpp
//////////////////////////////////////////////////////////////////////////////
// This file was borrowed from hd-1.7, and heavily modified. The original
// header comment block, containing copyright and other info, appears below.


/*****************************************************************************
*
*   Copyright P.J.Ruczynski 1990
*   This software is free for redistribution and re-use as long as this
*   copyright is included in all source files. This software is supplied
*   as is and no responsibilty is taken by the author for any problems
*   arising from this code.
*
* File name      -  hexprint.c
*
* Module name    -  HEXPRINT
*
* Author         -  P.J.Ruczynski    <pjr@pyra.co.uk>
*
* Second Release -  31 May 1990 (V1.6)
* First Release  -  16 Feb 1990 (V1.4)
*
* Version number -  1.6
*
* Description    -  A hexadecimal printing routine, will print the given
*                   buffer with the hex on the left and the ascii on the
*                   right.
*
*            Revision List
*
* pjr  08.05.89  Added offset numbering. This can be compiled out for
*                optimum performance.
*
* pjr  11.05.89  Added compressed output format. This can be compiled
*                out for optimum performance.
*
* pjr  02.08.89  Changed the bcopy and bcmp routines to use defines defined in
*                hexprint.h and controlled by compilation flag 'ATT'. With this
*                defined att routines are used, default means bsd routines are
*                used.  Note that the compression stuff uses BCMP and BCOPY.
*
* pjr  04.08.89  Added output to file capability. This can be compiled in
*                by defining the H_FILEIO flag. Note that you also have to have
*                a file open with an fd of linebuf, although this can be changed by
*                changing the decn below.
*
* pjr  15.05.90  Added octal and decimal output formatting.  This is
*                controlled by the fmt_ strings and the h_slen variabale which
*                governs the no of chars output on a line, the default for this
*                is equivalent to the base we are viewing in, eg octal is 8 etc.
*                Some preset values of can be found in hexprint.h.
*                The default is of course hex!
*
* grr  07.01.94  Added extra space after every 8th value.
*
*****************************************************************************/

#include <stdio.h>
#include "attrOLView.h"
#include "myItem.h"
#include "hexprint.h"



// hexprint
//
// routine to print a buffer out in hex form with ascii form on
// the right hand side
//***************************************************************************

void AttrOLView::Hexprint(
	const char *data,
	size_t datalen)
{
	int i,j;
	char string[H_SLEN];            // H_SLEN is 16 and max so ok with this
	char o_string[H_SLEN];          // string prior to the current one
	static char linebuf[1024];
	int lbo=0;                      // linebuf offset
	
	int done_compression = FALSE;   // have we done any compression ?
	
	long o;
	
	linebuf[0] = '\0';
		
	if (offset_print)
	{
		o = offset;
		sprintf(linebuf+lbo, fmt_offset, o);
		lbo = strlen(linebuf);  // NOT efficient!
	}
	
	for (i=0; i<datalen; i++)
	{
		string[i%h_slen] = *data++;
		
		
		if (i%h_slen == (h_slen-1))
		{
			o += h_slen;
			
			if (hex_compression)
			{
				if (i == (h_slen-1))  // first time around ?
					BCOPY(string, o_string, h_slen);
				else
				{
					if (BCMP(o_string, string, h_slen) == 0)
					{
						sprintf(linebuf+lbo,"* ");
						lbo+=2;
						done_compression = TRUE;
						BCOPY(string, o_string, h_slen);
						continue;
					}
					else if (done_compression)
					{
						//sprintf(linebuf,"\n");
						AddItem(new 
							MyStringItem(linebuf, CurLevel(), false));
						lbo=0;
						
						if (offset_print)
						{
							sprintf(linebuf+lbo, fmt_offset, o - h_slen);
							lbo = strlen(linebuf);
						}
						
						done_compression = FALSE;
					}
					BCOPY(string, o_string, h_slen);
				}
			}
			
			
			for (j=0; j<h_slen; j++)
			{
				sprintf(linebuf+lbo, fmt_dump, (unsigned char)string[j]);
				lbo = strlen(linebuf);
				if ((j+1)%8 == 0)
				{
					// sprintf(linebuf, " ");
					linebuf[lbo++] = ' ';
					linebuf[lbo]   = '\0';
				}
			}
			
			for (j=0; j<h_slen; j++)
			{
				linebuf[lbo++] =
					(string[j] >= 0x20 && string[j] <= 0x7e)
					? string[j] : '\1';  // ascii val 1 makes a box
			}
				
			linebuf[lbo]   = '\0';

			//sprintf(linebuf, "\n");
			AddItem(new MyStringItem(linebuf, CurLevel(), false));
			lbo=0;
						
			if ((i != (datalen-1)) && offset_print)
			{	
				sprintf(linebuf+lbo, fmt_offset, o);
				lbo = strlen(linebuf);
			}
			
			
		}
		else
		{
		
			if ((i%h_slen < (h_slen-1)) && (i == datalen-1))
			{
				
				if (done_compression)
				{
					AddItem(new MyStringItem(linebuf, CurLevel(), false));
					lbo=0;
					//sprintf(linebuf, "\n");
					
					if (offset_print)
					{
						sprintf(linebuf+lbo, fmt_offset, o);
						lbo = strlen(linebuf);
					}
					
					done_compression = false;
				}
				BCOPY(string, o_string, h_slen);
				
				for (j=0; j<=(i%h_slen); j++)
				{
					sprintf(linebuf+lbo, 
					        fmt_dump, (unsigned char)string[j]);
					lbo = strlen(linebuf);

					if ((j+1)%8 == 0)
						//sprintf(linebuf, " ");
						linebuf[lbo++] = ' ';
				}
				linebuf[lbo] = '\0';
				
				for (j=0; j<((h_slen-1)-(i%h_slen)); j++)
				{
					sprintf(linebuf+lbo, fmt_pad);    // PAD STRING
					lbo = strlen(linebuf);
					if ((j+(i%h_slen)+1)%8 == 0)
						//sprintf(linebuf, " ");
						linebuf[lbo++] = ' ';
				}
				linebuf[lbo++] = ' ';
				linebuf[lbo] = '\0';
				
				for (j=0; j<=(i%h_slen); j++)
					linebuf[lbo++] =
						(string[j] >= 0x20 && string[j] <= 0x7e)
						? string[j] : '\1';  // ascii val 1 makes a box
				linebuf[lbo]   = '\0';

				
				//sprintf(linebuf, "\n");
				AddItem(new MyStringItem(linebuf, CurLevel(), false));
				lbo=0;
			}
		}
	} // end of for
	
	
	if (done_compression)
	{	
		//sprintf(linebuf, "\n");
		AddItem(new MyStringItem(linebuf, CurLevel(), false));
		lbo=0;		
	}
	
} // end of hexprint



