//: main.cpp
//////////////////////////////////////////////////////////////////////////////
//
// This file provides the self-installation functionality.  For this to work,
// the add-on must be built as an Application, not a shared library. 
// Applications make fine shared libraries, and thus fine add-ons.


#include <Directory.h>
#include <Path.h>
#include <Entry.h>
#include <Alert.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "addOnControl.h"


#if (ADD_ON_NOT_APP) 

status_t do_install(bool sys_NOT_user, char * pathRet=0);


int main()
{
	new BApplication(
		"application/x-vnd.liquidorb-attributeViewerTrackerPlugIn");

	//MakeTestFile();

	BAlert * alert = new BAlert("alert", 
		"-- AttributeViewer--                   (v0.6 (alpha))\n "
		"This is a tracker add-on.  It is self installing.  \n\n"
		"<< NOTE: User Add-Ons "
		"for the tracker SHOULD go in user config space.  This "
		"is broken in PR, but may be fixed in PR2.  For PR, Tracker "
		"Add-ons will work ONLY in system space. >>  \n\n"
		"Install in...",
		"System space", "User Config space", "Cancel", B_WIDTH_FROM_WIDEST);



	int32 rval=-1;
	if ((rval = alert->Go()) <= 1)
	{
		char buf1[4096]="", buf2[2048]="";
		status_t status = do_install(rval==0, buf2);  
		
		if (status == B_OK)
			sprintf(buf1, 
				"AttributeViewer successfully installed in: \n"
				"\"%s\".",   buf2);
		else
			sprintf(buf1, 
				"Sorry, there was a problem installing the file: \n"
				"\"%s\".",     strerror(status));
				
		alert = new BAlert("alert", buf1, "OK", 0, 0, B_WIDTH_AS_USUAL,
			status==B_OK ?  B_IDEA_ALERT : B_WARNING_ALERT );
		alert->Go();
	}

	delete be_app;	
	return 0;
}


status_t do_install(bool sys_NOT_user, char * pathRet)
{
	status_t status = B_ERROR;

	BPath path;
	if (B_OK != (status = find_directory(
			(sys_NOT_user) ? B_BEOS_ADDONS_DIRECTORY 
			               : B_USER_ADDONS_DIRECTORY, 
			&path)))
		return status;
	
	path.Append("Tracker");
	
	if (B_OK != (status = create_directory(path.Path(), 0755)))
		return status;
	
		
	app_info info;
	if (B_OK != (status = be_app->GetAppInfo(&info)))
		return status;
	
	BEntry entry(&info.ref, true);
	if (B_OK != (status = 
			entry.MoveTo(&BDirectory(path.Path()), NULL, true)))
		return status;
	
	if (pathRet)
		strcpy (pathRet, path.Path());
		
	return B_OK;
}



#else   // stand alone app




#include "myWindow.h"

class MyApp : public BApplication
{

public:

	MyApp(const char * mime);

	virtual void RefsReceived(BMessage *message);
};

void mallochdlr(enum mcheck_status val);
const char * mallocProbEnumToStr(enum mcheck_status val);


const char * mallocProbEnumToStr(enum mcheck_status val)
{
	static char *mallocProbReasons [] =
	{
     "MCHECK_DISABLED",	/* = -1 Consistency checking is not turned on.  */
     "MCHECK_OK",		/* Block is fine.  */
     "MCHECK_FREE",		/* Block freed twice.  */
     "MCHECK_HEAD",		/* Memory before the block was clobbered.  */
     "MCHECK_TAIL"		/* Memory after the block was clobbered.  */
	};

	if (val<MCHECK_DISABLED || val>MCHECK_TAIL)
		return "<< INVALID VALUE!!! >>";
	else
		return mallocProbReasons[val+1];
} 



void mallochdlr(enum mcheck_status val)
{
	char buf[2048];
	sprintf(buf, "Malloc is hosed. Reason: \n%s ", 
		mallocProbEnumToStr(val));	
	BAlert * alert = new BAlert("POOF!!", buf,"[Explitive deleted]!");
	exit(val);
}

main()
{	
	//mcheck(&mallochdlr);   // use my func for MALLOC_DEBUG

	MyApp *myApplication;


	myApplication = new MyApp(
		"application/x-vnd.LiquidOrb-attributeViewer");
	myApplication->Run();
	
	delete(myApplication);
	return(0);
}

MyApp::MyApp(const char * mime)
		  : BApplication(mime)
{
puts("MyApp ctor");	
	// nothing!
}

void MyApp::RefsReceived(BMessage *message)
{
puts("MyApp::RefsReceived");	
	
	
	MyWindow * wind = new MyWindow(
	BRect(50,50, 590,400), "-- Attribute Viewer --");

	wind->Show();
	wind->SetRefs(message);
	wind->RefreshData();
}

#endif


 
/*/ TEMP CODE \/ \/ \/ 

void MakeTestFile();
void MakeTestFile()
{
	beep();beep();beep();
	popen("touch MyTestFile", "r");
	
	BNode node("MyTestFile"); 
	if (node.InitCheck() != B_NO_ERROR)
		return;
	
	char s[] = "If you can read this you are ◊∆ƒƒ¥†∂‡ÑÑ‡ ƒ too close!";
	size_t l = strlen(s);
	node.WriteAttr("any",B_ANY_TYPE, 0, s, l);
	node.WriteAttr("ascii",B_ASCII_TYPE, 0, s, l);
	bool b=true;
	node.WriteAttr("bool",B_BOOL_TYPE, 0, &b, sizeof(b));
	char ch='C';
	node.WriteAttr("char",B_CHAR_TYPE, 0, &ch, sizeof(ch));
	double d=PI;
	node.WriteAttr("double",B_DOUBLE_TYPE, 0, &d, sizeof(d));
	float f=PI/2;
	node.WriteAttr("float",B_FLOAT_TYPE, 0, &f, sizeof(f));
	int64 i64=-12456789764334434LL; int32 i32=-343223325; int16 i16=-16336; int8 i8=-121;
	node.WriteAttr("int64",B_INT64_TYPE, 0, &i64, 8);
	node.WriteAttr("int32",B_INT32_TYPE, 0, &i32, 4);
	node.WriteAttr("int16",B_INT16_TYPE, 0, &i16, 2);
	node.WriteAttr("int8", B_INT8_TYPE,  0,  &i8, 1);
	uint64 u64=12456789764334434LL; uint32 u32=343223325; uint16 u16=16336; uint8 u8=121;
	node.WriteAttr("uint64",B_UINT64_TYPE, 0, &u64, 8);
	node.WriteAttr("uint32",B_UINT32_TYPE, 0, &u32, 4);
	node.WriteAttr("uint16",B_UINT16_TYPE, 0, &u16, 2);
	node.WriteAttr("uint8", B_UINT8_TYPE,  0,  &u8, 1);
	BMessage msg;
	node.WriteAttr("msg", B_MESSAGE_TYPE, 0, &msg, sizeof(msg));
	node.WriteAttr("fake messenger", B_MESSENGER_TYPE, 0, s, l);
	node.WriteAttr("MIME", B_MIME_TYPE, 0, "image/jpeg", 10);
	node.WriteAttr("B_OBJECT_TYPE", B_OBJECT_TYPE, 0, s, l);
	off_t off=133434;
	node.WriteAttr("off_t",B_OFF_T_TYPE, 0, &off, sizeof(off));
	node.WriteAttr("path??",B_PATH_TYPE, 0, s, l);
	node.WriteAttr("pattern",B_PATTERN_TYPE, 0, &B_MIXED_COLORS, sizeof(B_MIXED_COLORS));
	node.WriteAttr("pattern BAD",B_PATTERN_TYPE, 0, s, l);
	void * ptr = &off;
	node.WriteAttr("ptr!?",B_POINTER_TYPE, 0, &ptr, sizeof(ptr));
	node.WriteAttr("raw data",B_RAW_TYPE, 0, s, l);
	BPoint pt(12,34);
	node.WriteAttr("point",B_POINT_TYPE, 0, &pt, sizeof(pt));
	BRect  r(56,78,12,34);
	node.WriteAttr("rect",B_RECT_TYPE, 0, &r, sizeof(r));
	BEntry entry("/boot/beos/system"); entry_ref ref; entry.GetRef(&ref);
	node.WriteAttr("entry_ref",B_REF_TYPE, 0, &ref, sizeof(ref));
	entry.SetTo("/this/is/crap"); entry.GetRef(&ref);
	node.WriteAttr("BAD entry_ref",B_REF_TYPE, 0, &ref, sizeof(ref));
	node.WriteAttr("MONO 1",B_MONOCHROME_1_BIT_TYPE, 0, s, l);
	node.WriteAttr("GREY 8",B_GRAYSCALE_8_BIT_TYPE, 0, s, l);
	node.WriteAttr("COLOR 8",B_COLOR_8_BIT_TYPE, 0, s, l);
	node.WriteAttr("RGB 32", B_RGB_32_BIT_TYPE, 0, s, l);
	// skip the icons, we know they work!
	rgb_color c; c.red=123; c.green=3535; c.blue=12; c.alpha=900;
	node.WriteAttr("RGBColor",B_RGB_COLOR_TYPE, 0, &c, sizeof(c));
	size_t size=-23354; ssize_t ssize=23354;
	node.WriteAttr("size_t",B_SIZE_T_TYPE,  0, &size,  sizeof(size));
	node.WriteAttr("ssize_t",B_SSIZE_T_TYPE, 0, &ssize, sizeof(ssize));
	node.WriteAttr("string",B_STRING_TYPE, 0, s, l+1);
	time_t t = time(0); bigtime_t bt = system_time();
	node.WriteAttr("time",B_TIME_TYPE, 0, &t, sizeof(t));
	node.WriteAttr("bigtime_t",B_TIME_TYPE, 0, &bt, sizeof(bt));

}
 
// TEMP CODE /\ /\ /\ 
// */



