//: imageItem.cpp
//////////////////////////////////////////////////////////////////////////////

#include "myItem.h"
#include <View.h>
#include <stdio.h>



//////////////////////////////////////////////////////////////////////////////
// class MyStringItem
/*
	Handles text with simple embedded format codes. These are the format 
	code byte (as define in the header file), followed by a single character
	which dermines the new format. These are:
		p -- be_plain_font
		b -- be_bold_font
		f -- be_fixed_font  (the default font)
		_ -- toggle underlining (NOT IMPLEMENTED!)
		g -- toggle gray (default is black)
		D -- set back to defaults
	Note that this class is designed to provide some useful formatting 
	functionality, NOT to provide the full text formatting capability of 
	the BeOS. 
	Also, all '\1' characters are printed as gray rectangles.
*/


MyStringItem::MyStringItem(
 	const char *text, uint32 outlineLevel, bool expanded)
 : BStringItem(text, outlineLevel, expanded),  // use formatted FOR NOW!!
   _ascent(0),
   _totalHeight(0),
   _totalWidth(0),
   _fmtText(0)   
{
	_fmtText = new char[strlen(text)+1];
	strcpy(_fmtText, text);	
}


MyStringItem::~MyStringItem()
{
	delete [] _fmtText;
}


void MyStringItem::Update(BView * /*owner*/, const BFont * /*font*/)
{
	bool plainUsed=false, boldUsed=false, fixedUsed=false;
	const BFont * curFont = be_fixed_font;
	char fontDesig='f';

//printf("UPDATE \\/ \\/ \\/ : %s\n", _fmtText);
//printf("UPDATE: %s\n", owner->Window()->IsLocked() ? "LOCKED" : "**NOT** LOCKED");

	_totalWidth = 0;

	bool eosFound=false;
	uint ii, fmtStart=0, fmtEnd=0;
	
	for (ii=0;; ii++)
	{
		if ( _fmtText[ii] == MSI_FORMAT_CODE_BYTE  || 
		     _fmtText[ii] == '\0'                     )
		{
			fmtEnd=ii;  // this is redundant, but self documenting
			eosFound = (_fmtText[ii] == '\0');
			
			if (fmtStart < fmtEnd)  // if there is new text
			{
				_totalWidth += 
					curFont->StringWidth(
						&_fmtText[fmtStart], fmtEnd-fmtStart);
				
				// now the we have counted text in this font, mark as used
				switch(fontDesig)
				{
					case 'p': plainUsed = true;   break;
					case 'b': boldUsed  = true;   break;
					case 'f': fixedUsed = true;   break;
				}
			}

			if (!eosFound) // its a format specifier, not '\0'
			{
				switch (_fmtText[++ii])
				{
				  case '\0':
				  {
				  	goto L_endOfForLoop;  // goto rules!!! You like it! ;->
				  } break;
				  
				  case 'p':
				  {
				  	curFont = be_plain_font;
				  	fontDesig='p';
				  } break;
				  
				  case 'b':
				  {
				  	curFont = be_bold_font;
				  	fontDesig='b';
				  } break;
				  
				  case 'D':  // default
				  case 'f':
				  {
				  	curFont = be_fixed_font;
				  	fontDesig='f';
				  } break;
				  
				  default:
				  	break; // ignore others, only these affect width/height
				}
			}
			else
				break;
			
			fmtStart = fmtEnd+2; 
		}
	
	}	
	L_endOfForLoop: // only goto here if '\0' found after FORMAT_CODE_BYTE

	font_height fontHt;
	
	if (plainUsed)
	{	be_plain_font->GetHeight(&fontHt);
		_ascent = fontHt.ascent;
		_totalHeight = _ascent + fontHt.descent ;//+ fontHt.leading;		
	} 
	
	if (boldUsed)
	{	be_bold_font->GetHeight(&fontHt);
		_ascent = max_c(_ascent, fontHt.ascent);
		_totalHeight =  max_c(_totalHeight, 
				fontHt.ascent + fontHt.descent );//+ fontHt.leading);
	} 
	
	if (fixedUsed)
	{	be_fixed_font->GetHeight(&fontHt);
		_ascent = max_c(_ascent, fontHt.ascent);
		_totalHeight =  max_c(_totalHeight, 
				fontHt.ascent + fontHt.descent );//+ fontHt.leading);
	};
	

	SetWidth  (_totalWidth);
	SetHeight (_totalHeight);

/*
printf("P%d B%d F%d  W%f H%f  === %s\n", 
(int)plainUsed, (int)boldUsed, (int)fixedUsed, 
_totalWidth, _totalHeight, _fmtText);
*/
//printf("UPDATE /\\ /\\ /\\ : %s\n\n", _fmtText);

}




void  MyStringItem::DrawItem(BView *owner, BRect frame, bool complete)
{
//printf("DRAW \\/ \\/ \\/ : %s\n", _fmtText);
//printf("DRAW: %s\n", owner->Window()->IsLocked() ? "LOCKED":"**NOT** LOCKED");
	//BAutoLock lockWin(Window);

	// snag old values 
	rgb_color oldColor = owner->HighColor();
	BFont oldFont;
	owner->GetFont(&oldFont);

	// do background
	if ( IsSelected() )
	{
		owner->SetHighColor(200,200,200);
		owner->FillRect(frame);
	}
	else if (complete)
	{	
		owner->SetHighColor(owner->ViewColor());
		owner->FillRect(frame);
	}
	


	// now draw the string!

	const BFont *  curFont = be_fixed_font;
	const BFont * lastFont = 0;
	
	// set defaults
	owner->SetFont(curFont);
	owner->SetHighColor(0,0,0);
	
	owner->MovePenTo(frame.left, frame.top+_ascent);


	bool eosFound=false, isGray=false;
	uint ii, fmtStart=0, fmtEnd=0;
	
	for (ii=0;; ii++)
	{
		if ( _fmtText[ii] == MSI_FORMAT_CODE_BYTE  || 
		     _fmtText[ii] == '\1'                  ||
		     _fmtText[ii] == '\0'                     )
		{
			fmtEnd=ii;  // this may be redundant, but self documenting
			eosFound = (_fmtText[ii] == '\0');

						
			if (fmtStart < fmtEnd)  // if there is new text
			{
				if (lastFont != curFont)
					owner->SetFont(curFont);
					
				owner->DrawString(&_fmtText[fmtStart], fmtEnd-fmtStart);
			}

			if (_fmtText[ii] == '\1')  // print as gray dot
			{
				if (lastFont != curFont)
					owner->SetFont(curFont);
				
				owner->SetHighColor(128,128,128);	
				owner->DrawChar('.');
			  	if (!isGray)
					owner->SetHighColor(0,0,0);
				fmtStart = fmtEnd+1; 	
			}
			else if (_fmtText[ii] == MSI_FORMAT_CODE_BYTE) 
			{
				switch (_fmtText[++ii])
				{
				  case 'p':
				  {
				  	lastFont = curFont;
				  	curFont = be_plain_font;
				  } break;
				  
				  case 'b':
				  {
				  	lastFont = curFont;
				  	curFont = be_bold_font;
				  } break;
				  
				  case 'f':
				  {
				  	lastFont = curFont;
				  	curFont = be_fixed_font;
				  } break;
				  
				  case 'g':
				  {
				  	isGray = !isGray;
				  	if (isGray)
						owner->SetHighColor(128,128,128);
					else
						owner->SetHighColor(0,0,0);
				  } break;
				  
				  case 'D':
				  {
				  	lastFont = curFont;
				  	curFont = be_fixed_font;
					owner->SetHighColor(0,0,0);
				  } break;
				  
				  case '\0':
				  {
				  	goto L_endOfForLoop;  // goto rules!!! You like it! ;->
				  } break;
				  

				  default:
				  	break; // ignore others
				}
				
				fmtStart = fmtEnd+2; 	
			}
			else if (eosFound)
				break;
		}
	}	
	L_endOfForLoop: // only goto here if '\0' found after FORMAT_CODE_BYTE
	
	owner->SetFont(&oldFont);
	owner->SetHighColor(oldColor);

//printf("DRAW /\\ /\\ /\\ : %s\n\n", _fmtText);

}



//////////////////////////////////////////////////////////////////////////////

ImageItem::ImageItem(
 	float width,
 	float height,
 	color_space mode,
 	const void * data,
 	uint32 outlineLevel,
 	bool expanded)
 : BListItem(outlineLevel, expanded),
   _width(width),
   _height(height),
   _bitmap(0)
{
//puts("ImageItem IN");
//snooze(1);
 	_bitmap = new BBitmap(
 		BRect(0,0, width-1, height-1),
 		mode);
 		
 	// We are assuming for now that the data is 32bit alligned.
 	// otherwise, the bug in this assumption is made manifest!	
 	// This doesn't mean the starting address must be aligned, but
 	// that the row length (for 8bit) is a multiple of 4 bytes.
 	// Right now, we only get built for icons, which are always
 	// either 16x16 or 32x32.
 	_bitmap->SetBits(data, width*height, 0, mode);
//puts("ImageItem OUT");
}


ImageItem::~ImageItem()
{
	delete _bitmap;
}


void ImageItem::Update(BView * /*owner*/, const BFont * /*font*/)
{
	// 3 pixels on all sides!
	SetWidth  (_width+6);
 	SetHeight (_height+6); 	
}


void ImageItem::DrawItem(
	BView *owner,
	BRect bounds,
	bool complete)
{
	rgb_color oldColor = owner->HighColor();

	if ( IsSelected() )
	{
		owner->SetHighColor(200,200,200);
		owner->FillRect(bounds);
	}
	else if (complete)
	{	
		owner->SetHighColor(owner->ViewColor());
		owner->FillRect(bounds);
	}
	
	
	float oldPenSize = owner->PenSize();
	owner->SetPenSize(1);
	owner->SetHighColor(0,0,0);
	owner->StrokeRect(
		BRect(
			bounds.left+1,        bounds.top+1, 
			bounds.left+_width+4, bounds.top+_height+4));	
	owner->SetHighColor(oldColor);
	owner->SetPenSize(oldPenSize);
	
	owner->DrawBitmapAsync(
		_bitmap, BPoint(bounds.left+3, bounds.top+3));
}


//////////////////////////////////////////////////////////////////////////////


TitleItem::TitleItem(
	const char *text, 
	uint32 outlineLevel, 
	bool expanded)
 : BStringItem(text, outlineLevel, expanded)
{	
	be_bold_font->GetHeight(&_fontHt);
}


TitleItem::~TitleItem()
{
}


void TitleItem::DrawItem(BView *owner, BRect frame, bool complete)
{
	rgb_color oldColor = owner->HighColor();

	if ( IsSelected() )
	{
		owner->SetHighColor(200,200,200);
		owner->FillRect(frame);
	}
	else if (complete)
	{	
		owner->SetHighColor(owner->ViewColor());
		owner->FillRect(frame);
	}
	
	
	float oldPenSize = owner->PenSize();
	BFont oldFont;
	owner->GetFont(&oldFont);
	
	owner->SetHighColor(96,96,96);
	owner->SetPenSize(1);
	owner->StrokeLine(
		BPoint(0,frame.top), BPoint(frame.right, frame.top));

	owner->SetHighColor(0,0,0);
	owner->SetFont(be_bold_font);
	owner->DrawString(
		Text(), BPoint(frame.left+2, frame.top+_fontHt.ascent+1));

	owner->SetFont(&oldFont);
	owner->SetHighColor(oldColor);
	owner->SetPenSize(oldPenSize);
	
}


void TitleItem::Update(BView * /*owner*/, const BFont * /*font*/)
{
	// we ignore the chosen font, and use b_bold_font
	SetWidth(be_bold_font->StringWidth(Text()));
	be_bold_font->GetHeight(&_fontHt);
	SetHeight(_fontHt.ascent + _fontHt.leading + _fontHt.descent+2);
}




