//: myOLView.cpp
//////////////////////////////////////////////////////////////////////////////

#include <Window.h>
#include <Rect.h>
#include <View.h>
#include <Menu.h>
#include <ScrollView.h>

#include "myOLView.h"
#include "myWindow.h"

MyOLView::MyOLView(BRect rect)
 :	BOutlineListView (
		rect, "OutlineView",
		B_SINGLE_SELECTION_LIST, B_FOLLOW_ALL, 
		B_FRAME_EVENTS | B_WILL_DRAW | B_NAVIGABLE ),
	_curLevel(0)
{
}


MyOLView::~MyOLView()
{
}


void MyOLView::Refresh()
{
}


void MyOLView::MessageReceived(BMessage *msg)
{
	switch(msg->what)
	{	
	  case MY_MENU_ITEM_SELECTED:
	  {
		int32 key = -1;
     	msg->FindInt32("key", &key);              
		switch (key)
		{
		  case MyWindow::MENU_refresh:
		  {
			Refresh();
			
		  	BListItem * item = 
		  		ItemAt(CurrentSelection());
		  	if (!item)
		  		break;
		  	printf("H%f W%f\n", item->Height(), item->Width());
		  }	break;
		  	
		  case MyWindow::MENU_collapseAll:
		  	SetExpandedOnAll(false);
		  	break;
		  	
		  case MyWindow::MENU_expandAll:
		  	SetExpandedOnAll(true);
		  	break;
		  
		  case MyWindow::MENU_copyLine:	
		  	CopyLine(true);
		  	break;	

		  case MyWindow::MENU_copyValue:	
		  	CopyLine(false);
		  	break;	
		}		

	  } break;
	
	  default:
		inherited::MessageReceived(msg);
	}
}




void MyOLView::SetExpandedOnAll(bool expanded)
{
	if (!Window)
		return;   // just in case
		
	Window()->Lock();

	long max = FullListCountItems();
	if (expanded)
		for (long ii=0; ii < max; ii++)
			Expand(FullListItemAt(ii));	
	else
		for (long ii=0; ii < max; ii++)
			Collapse(FullListItemAt(ii));	
	
	Window()->Unlock();
}


void MyOLView::CopyLine(bool full)
{
	BStringItem * item = dynamic_cast<BStringItem*>
		(ItemAt(CurrentSelection()));
	const char * text=0; 
	if (!item || !(text = item->Text()))
	{
		beep();
		return;
	}
	
	if ( be_clipboard->Lock() ) 
	{ 
		be_clipboard->Clear(); 
		BMessage *clip = be_clipboard->Data(); 
		int32 len=0;
		//clip->PrintToStream();
		
		int valOffset = !full ? VAL_START_POS : 0;
		if ( clip->AddData(
				"text/plain", B_MIME_TYPE, 
				text         + valOffset, 
				strlen(text) - valOffset  )  == B_OK) 
			be_clipboard->Commit(); 
		else	
			beep(); // don't commit (duh)
				
		be_clipboard->Unlock(); 
	}		
}


void MyOLView::DeleteAllAndMakeEmpty()
{
	BListItem * item=0;
	int32 ii=0;
	while (!!(item = FullListItemAt(ii++)))
		delete item;
	MakeEmpty();
}






